@@ -131,6 +131,7 @@ class Program:
 
   def extract_cpp(self) -> 'Program':
     """Extract HLS C++ files."""
+    _logger.info('extracting HLS C++ files')
     for task in self._tasks.values():
       with open(self.get_cpp(task.name), 'w') as src_code:
         src_code.write(util.clang_format(task.code))
@@ -144,6 +145,7 @@ class Program:
   def run_hls(self, clock_period: Union[int, float, str],
               part_num: str) -> 'Program':
     """Run HLS with extracted HLS C++ files and generate tarballs."""
+    _logger.info('running HLS')
 
     def worker(task: Task) -> Iterator[None]:
       with open(self.get_tar(task.name), 'wb') as tarfileobj:
@@ -154,6 +156,7 @@ class Program:
             clock_period=clock_period,
             part_num=part_num,
             auto_prefix=True,
+            hls='vitis_hls',
         ) as proc:
           yield
           stdout, stderr = proc.communicate()
@@ -169,6 +172,7 @@ class Program:
 
   def extract_rtl(self) -> 'Program':
     """Extract HDL files from tarballs generated from HLS."""
+    _logger.info('extracting RTL files')
     for task in self._tasks.values():
       with tarfile.open(self.get_tar(task.name), 'r') as tarfileobj:
         tarfileobj.extractall(path=self.work_dir)
@@ -189,15 +193,19 @@ class Program:
         Program: Return self.
     """
     # extract and parse RTL and populate tasks
+    _logger.info('parsing RTL files and populating tasks')
     oldcwd = os.getcwd()
     os.chdir(self.work_dir)
     for task in self._tasks.values():
+      _logger.debug('parsing %s', task.name)
       task.module = rtl.Module([self.get_rtl(task.name)])
+      _logger.debug('populating %s', task.name)
       self._populate_task(task)
     os.chdir(oldcwd)
 
     # generate partitioning constraints if partitioning directive is given
     if directive is not None:
+      _logger.info('generating partitioning constraints')
       floorplan = directive.get('floorplan')
       if floorplan is None:
         floorplan = autobridge.generate_floorplan(
@@ -212,6 +220,7 @@ class Program:
       self.top_task.module.register_level = register_level
 
     # instrument the upper-level RTL
+    _logger.info('instrumenting upper-level RTL')
     for task in self._tasks.values():
       if task.is_upper:
         task.module.cleanup()
@@ -227,6 +236,7 @@ class Program:
         with open(self.get_rtl(task.name), 'w') as rtl_code:
           rtl_code.write(task.module.code)
 
+    _logger.info('writing RTL files')
     for name, content in self.tcl_files.items():
       with open(os.path.join(self.rtl_dir, name + '.tcl'), 'w') as tcl_file:
         tcl_file.write(content)
@@ -244,6 +254,7 @@ class Program:
     return self
 
   def pack_rtl(self, output_file: BinaryIO) -> 'Program':
+    _logger.info('packaging RTL code')
     rtl.pack(top_name=self.top,
              ports=self.toplevel_ports,
              rtl_dir=self.rtl_dir,
@@ -255,6 +266,8 @@ class Program:
       directive: Dict[str, List[Union[str, Dict[str, List[str]]]]],
       constraints: TextIO,
   ) -> None:
+    _logger.info('processing partition directive')
+
     # mapping instance names to region names
     instance_dict = {self.ctrl_instance_name: ''}
 
@@ -340,6 +353,7 @@ class Program:
         for idx, obj in enumerate(objs))
 
   def _connect_fifos(self, task: Task) -> None:
+    _logger.debug("  connecting %s's children tasks", task.name)
     for fifo_name, fifo in task.fifos.items():
       directions = {
           'consumed_by': rtl.ISTREAM_SUFFIXES,
@@ -363,10 +377,8 @@ class Program:
 
           # if this FIFO is not declared, connect it directly to ports
           if 'depth' not in fifo:
-            port_suffix, port_direction = rtl.STREAM_PORT_MAPPING[suffix]
-            port_name = fifo_name + port_suffix
-            port = ast.Wire(name=port_name, width=wire_width)
-            task.module.add_signals([port])
+            port_name = rtl.fifo_port_name(fifo_name, suffix)
+            port_direction = rtl.STREAM_PORT_DIRECTION[suffix]
             if port_direction == 'input':
               task.module.add_logics([
                   ast.Assign(left=ast.Identifier(wire_name),
@@ -379,10 +391,20 @@ class Program:
               ])
 
   def _instantiate_fifos(self, task: Task) -> None:
-    for fifo_name, fifo in task.fifos.items():
-      # skip instantiating if the fifo is not declared in this task
-      if 'depth' not in fifo:
-        continue
+    _logger.debug('  instantiating FIFOs in %s', task.name)
+
+    # skip instantiating if the fifo is not declared in this task
+    fifos = {name: fifo for name, fifo in task.fifos.items() if 'depth' in fifo}
+    if not fifos:
+      return
+
+    col_width = max(
+        max(len(name), len(util.get_instance_name(fifo['consumed_by'])),
+            len(util.get_instance_name(fifo['produced_by'])))
+        for name, fifo in fifos.items())
+
+    for fifo_name, fifo in fifos.items():
+      _logger.debug('    instantiating %s.%s', task.name, fifo_name)
 
       # add FIFO instances
       task.module.add_fifo_instance(
@@ -393,10 +415,6 @@ class Program:
 
       # print debugging info
       debugging_blocks = []
-      col_width = max(
-          max(len(name), len(util.get_instance_name(fifo['consumed_by'])),
-              len(util.get_instance_name(fifo['produced_by'])))
-          for name, fifo in task.fifos.items())
       fmtargs = {
           'fifo_prefix': '\\033[97m',
           'fifo_suffix': '\\033[0m',
@@ -668,52 +686,59 @@ class Program:
         ast.Reg(countdown.name, width=ast.make_width(countdown_width)),
     ])
 
-    global_fsm = [
-        ast.make_if_with_block(
-            cond=is_state(STATE00),
-            true=ast.make_if_with_block(
-                cond=self.start_q[-1],
-                true=set_state(STATE01),
+    state01_action = set_state(STATE10)
+    if is_done_signals:
+      state01_action = ast.make_if_with_block(
+          cond=ast.make_operation(
+              operator=ast.Land,
+              nodes=(x[-1] for x in reversed(is_done_signals)),
+          ),
+          true=state01_action,
+      )
+
+    global_fsm = ast.make_case_with_block(
+        comp=rtl.STATE,
+        cases=[
+            (
+                STATE00,
+                ast.make_if_with_block(
+                    cond=self.start_q[-1],
+                    true=set_state(STATE01),
+                ),
             ),
-        ),
-        ast.make_if_with_block(
-            cond=is_state(STATE01),
-            true=ast.make_if_with_block(
-                cond=ast.make_operation(
-                    operator=ast.Land,
-                    nodes=(x[-1] for x in reversed(is_done_signals)),
-                ),
-                true=set_state(STATE10),
+            (
+                STATE01,
+                state01_action,
             ),
-        ),
-        ast.make_if_with_block(
-            cond=is_state(STATE10),
-            true=ast.make_block([
-                set_state(STATE11 if self.register_level else STATE00),
-                ast.NonblockingSubstitution(
-                    left=countdown,
-                    right=ast.make_int(max(0, self.register_level - 1)),
-                ),
-            ]),
-        ),
-        ast.make_if_with_block(
-            cond=is_state(STATE11),
-            true=ast.make_if_with_block(
-                cond=ast.Eq(
-                    left=countdown,
-                    right=ast.make_int(0, width=countdown_width),
-                ),
-                true=set_state(STATE00),
-                false=ast.NonblockingSubstitution(
-                    left=countdown,
-                    right=ast.Minus(
+            (
+                STATE10,
+                [
+                    set_state(STATE11 if self.register_level else STATE00),
+                    ast.NonblockingSubstitution(
                         left=countdown,
-                        right=ast.make_int(1, width=countdown_width),
+                        right=ast.make_int(max(0, self.register_level - 1)),
+                    ),
+                ],
+            ),
+            (
+                STATE11,
+                ast.make_if_with_block(
+                    cond=ast.Eq(
+                        left=countdown,
+                        right=ast.make_int(0, width=countdown_width),
+                    ),
+                    true=set_state(STATE00),
+                    false=ast.NonblockingSubstitution(
+                        left=countdown,
+                        right=ast.Minus(
+                            left=countdown,
+                            right=ast.make_int(1, width=countdown_width),
+                        ),
                     ),
                 ),
             ),
-        ),
-    ]
+        ],
+    )
 
     task.module.add_logics([
         ast.Always(
@@ -722,7 +747,7 @@ class Program:
                 ast.make_if_with_block(
                     cond=rtl.RST,
                     true=set_state(STATE00),
-                    false=ast.make_block(global_fsm),
+                    false=global_fsm,
                 )),
         ),
         ast.Assign(left=rtl.IDLE, right=is_state(STATE00)),
