@@ -437,9 +437,9 @@ class Module:
     self.del_params(prefix='ap_ST_fsm_state')
     self.del_signals(prefix='ap_CS_fsm')
     self.del_signals(prefix='ap_NS_fsm')
-    self.del_signals(suffix='_fifo_V_read')
-    self.del_signals(suffix='_fifo_V_write')
-    self.del_signals(suffix='_fifo_V_blk_n')
+    self.del_signals(suffix='_read')
+    self.del_signals(suffix='_write')
+    self.del_signals(suffix='_blk_n')
     self.del_signals(HANDSHAKE_RST)
     self.del_signals(HANDSHAKE_DONE)
     self.del_signals(HANDSHAKE_IDLE)
